@@ -773,13 +773,7 @@ bool YFrameWindow::handleKey(const XKeyEvent &key) {
             } else if (gKeyWinTileCenter == key) {
                 wmTile(actionTileCenter);
             } else if (gKeyWinSmartPlace == key) {
-                if (canMove()) {
-                    int newX = x();
-                    int newY = y();
-                    if (manager->getSmartPlace(true, this, newX, newY, width(), height(), getScreen())) {
-                        setCurrentPositionOuter(newX, newY);
-                    }
-                }
+                wmSmartPlace();
             } else if (isIconic() || isRollup()) {
                 KeySym k = keyCodeToKeySym(key.keycode);
                 unsigned m = KEY_MODMASK(key.state);
@@ -970,6 +964,10 @@ void YFrameWindow::endMoveSize() {
         if (!raiseOnClickClient || !canRaise() || !overlapped())
             container()->releaseButtons();
     }
+    if (doNotCover() && isManaged() &&
+        YRect(origX, origY, origW, origH) != geometry()) {
+        manager->updateWorkArea();
+    }
 
     if (taskBar) {
         taskBar->workspacesRepaint(getWorkspace());
@@ -1035,19 +1033,6 @@ bool YFrameWindow::handleBeginDrag(const XButtonEvent &down, const XMotionEvent
 }
 
 void YFrameWindow::moveWindow(int newX, int newY) {
-/// TODO #warning "reevaluate if this is legacy"
-#if 0
-    if (!doNotCover()) {
-        int mx, my, Mx, My;
-        manager->getWorkArea(this, &mx, &my, &Mx, &My);
-
-        newX = clamp(newX, (int)(mx + borderX() - width()),
-                           (int)(Mx - borderX()));
-        newY = clamp(newY, (int)(my + borderY() - height()),
-                           (int)(My - borderY()));
-    }
-#endif
-
     if (opaqueMove)
         drawMoveSizeFX(x(), y(), width(), height());
 
