@@ -9,6 +9,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <ctype.h>
 #include <regex.h>
 #include <new>
 #include "base.h"
@@ -16,6 +17,7 @@
 
 void MStringRef::alloc(size_t len) {
     fStr = new (malloc(sizeof(MStringData) + len + 1)) MStringData();
+    fStr->fLen = unsigned(len);
 }
 
 void MStringRef::create(const char* str, size_t len) {
@@ -33,22 +35,23 @@ void MStringRef::create(const char* str, size_t len) {
 }
 
 mstring::mstring(const MStringRef& str, size_t offset, size_t count):
-    fRef(str),
-    fOffset(offset),
-    fCount(count)
+    fRef(str)
 {
+    if (fRef && (offset || count < length())) {
+        fRef.create(&str[offset], count);
+    }
     acquire();
 }
 
 mstring::mstring(const char* str1, size_t len1, const char* str2, size_t len2):
-    fRef(len1 + len2), fOffset(0), fCount(len1 + len2)
+    fRef(len1 + len2)
 {
     if (fRef) {
         if (len1)
             strncpy(fRef->fStr, str1, len1);
         if (len2)
             strncpy(fRef->fStr + len1, str2, len2);
-        fRef[fCount] = 0;
+        fRef[len1 + len2] = 0;
         fRef.acquire();
     }
 }
@@ -59,7 +62,7 @@ mstring::mstring(const char* str):
 }
 
 mstring::mstring(const char* str, size_t len):
-    fRef(str, len), fOffset(0), fCount(len)
+    fRef(str, len)
 {
     acquire();
 }
@@ -73,13 +76,12 @@ mstring::mstring(const char *str1, const char *str2, const char *str3) {
     size_t len1 = str1 ? strlen(str1) : 0;
     size_t len2 = str2 ? strlen(str2) : 0;
     size_t len3 = str3 ? strlen(str3) : 0;
-    fOffset = 0;
-    fCount = len1 + len2 + len3;
-    fRef.alloc(fCount);
+    size_t count = len1 + len2 + len3;
+    fRef.alloc(count);
     if (len1) memcpy(fRef->fStr, str1, len1);
     if (len2) memcpy(fRef->fStr + len1, str2, len2);
     if (len3) memcpy(fRef->fStr + len1 + len2, str3, len3);
-    fRef[fCount] = 0;
+    fRef[count] = 0;
     fRef.acquire();
 }
 
@@ -93,9 +95,7 @@ mstring::mstring(const char *str1, const char *str2, const char *str3,
     for (int i = 0; i < count; ++i) {
         total += length[i] = (string[i] ? strlen(string[i]) : 0);
     }
-    fOffset = 0;
-    fCount = total;
-    fRef.alloc(fCount);
+    fRef.alloc(total);
     size_t build = 0;
     for (int i = 0; i < count; ++i) {
         if (length[i]) {
@@ -103,15 +103,14 @@ mstring::mstring(const char *str1, const char *str2, const char *str3,
             build += length[i];
         }
     }
-    fRef[fCount] = 0;
+    fRef[total] = 0;
     fRef.acquire();
 }
 
-mstring::mstring(long n):
-    fRef(size_t(23)), fOffset(0), fCount(0)
-{
-    snprintf(fRef->fStr, 23, "%ld", n);
-    fCount = strlen(fRef->fStr);
+mstring::mstring(long n) {
+    char buf[24];
+    snprintf(buf, 24, "%ld", n);
+    fRef.create(buf, strlen(buf));
     fRef.acquire();
 }
 
@@ -130,20 +129,18 @@ mstring& mstring::operator=(const mstring& rv) {
         fRef = rv.fRef;
         acquire();
     }
-    fOffset = rv.fOffset;
-    fCount = rv.fCount;
     return *this;
 }
 
 mstring mstring::substring(size_t pos) const {
-    return pos <= length()
-        ? mstring(fRef, fOffset + pos, fCount - pos)
+    return pos < length()
+        ? mstring(fRef, pos, length() - pos)
         : null;
 }
 
 mstring mstring::substring(size_t pos, size_t len) const {
-    return pos <= length()
-        ? mstring(fRef, fOffset + pos, min(len, fCount - pos))
+    return pos < length() && len
+        ? mstring(fRef, pos, min(len, length() - pos))
         : null;
 }
 
@@ -177,14 +174,24 @@ int mstring::charAt(int pos) const {
     return size_t(pos) < length() ? data()[pos] : -1;
 }
 
-bool mstring::startsWith(const mstring &s) const {
-    return s.isEmpty() || (s.length() <= length()
-        && 0 == memcmp(data(), s.data(), s.length()));
+bool mstring::startsWith(const char* s) const {
+    size_t slen = strlen(s);
+    return slen == 0 || (slen <= length()
+        && 0 == memcmp(data(), s, slen));
 }
 
-bool mstring::endsWith(const mstring &s) const {
-    return s.isEmpty() || (s.length() <= length()
-        && 0 == memcmp(data() + length() - s.length(), s.data(), s.length()));
+bool mstring::endsWith(const char* s) const {
+    size_t slen = strlen(s);
+    return slen == 0 || (slen <= length()
+        && 0 == memcmp(data() + length() - slen, s, slen));
+}
+
+int mstring::find(const char* str) const {
+    size_t len = strlen(str);
+    const char* found = (len == 0 || isEmpty()) ? nullptr :
+        static_cast<const char*>(memmem(
+                data(), length(), str, len));
+    return found ? int(found - data()) : (len ? -1 : 0);
 }
 
 int mstring::find(const mstring &str) const {
@@ -196,13 +203,13 @@ int mstring::find(const mstring &str) const {
 
 int mstring::indexOf(char ch) const {
     const char *str = isEmpty() ? nullptr :
-        static_cast<const char *>(memchr(data(), ch, fCount));
+        static_cast<const char *>(memchr(data(), ch, length()));
     return str ? int(str - data()) : -1;
 }
 
 int mstring::lastIndexOf(char ch) const {
     const char *str = isEmpty() ? nullptr :
-        static_cast<const char *>(memrchr(data(), ch, fCount));
+        static_cast<const char *>(memrchr(data(), ch, length()));
     return str ? int(str - data()) : -1;
 }
 
@@ -244,11 +251,11 @@ int mstring::compareTo(const mstring &s) const {
 
 bool mstring::copyTo(char *dst, size_t len) const {
     if (len > 0) {
-        size_t copy = min(len - 1, fCount);
+        size_t copy = min(len - 1, length());
         if (copy) memcpy(dst, data(), copy);
         dst[copy] = 0;
     }
-    return fCount < len;
+    return length() < len;
 }
 
 mstring mstring::replace(int pos, int len, const mstring &insert) const {
@@ -278,17 +285,19 @@ mstring mstring::searchAndReplaceAll(const mstring& s, const mstring& r) const {
 }
 
 mstring mstring::lower() const {
-    mstring mstr(nullptr, fCount);
-    for (size_t i = 0; i < fCount; ++i) {
-        mstr.fRef[i] = ASCII::toLower(data()[i]);
+    const size_t len = length();
+    mstring mstr(nullptr, len);
+    for (size_t i = 0; i < len; ++i) {
+        mstr.fRef[i] = tolower((unsigned char) data()[i]);
     }
     return mstr;
 }
 
 mstring mstring::upper() const {
-    mstring mstr(nullptr, fCount);
-    for (size_t i = 0; i < fCount; ++i) {
-        mstr.fRef[i] = ASCII::toUpper(data()[i]);
+    const size_t len = length();
+    mstring mstr(nullptr, len);
+    for (size_t i = 0; i < len; ++i) {
+        mstr.fRef[i] = toupper((unsigned char) data()[i]);
     }
     return mstr;
 }
@@ -309,15 +318,14 @@ const char* mstring::c_str()
     if (isEmpty()) {
         return "";
     }
-    else if (data()[fCount]) {
+    else if (data()[length()]) {
         if (fRef->fRefCount == 1) {
-            fRef[fOffset + fCount] = '\0';
+            fRef[length()] = '\0';
         } else {
             const char* str = data();
             fRef.release();
-            fRef.create(str, fCount);
+            fRef.create(str, length());
             fRef.acquire();
-            fOffset = 0;
         }
     }
     return data();
@@ -369,10 +377,10 @@ void mstring::fmt(const char* fmt, ...) {
         if (*s == '%' && s[1]) {
             s++;
             if (*s == 's') {
-                len += strlen(va_arg(ap, const char *));
+                len = len + strlen(va_arg(ap, const char *)) - 2;
             }
             if (*s == 'm') {
-                len += strlen(strerror(en));
+                len = len + strlen(strerror(en)) - 2;
             }
         }
     }
@@ -404,8 +412,6 @@ void mstring::fmt(const char* fmt, ...) {
     va_end(ap);
     release();
     fRef = ref;
-    fOffset = 0;
-    fCount = i;
     acquire();
 }
 
