@@ -1,7 +1,7 @@
-#ifndef __WMBUTTON_H
-#define __WMBUTTON_H
+#ifndef WMBUTTON_H
+#define WMBUTTON_H
 
-#include "yactionbutton.h"
+#include "ybutton.h"
 
 class YFrameWindow;
 class YFrameTitleBar;
@@ -24,6 +24,7 @@ public:
     virtual void actionPerformed(YAction action, unsigned int modifiers);
     void setKind(char kind);
     virtual void updatePopup();
+    virtual void updateToolTip();
 
     ref<YPixmap> getPixmap(int pn) const;
     YFrameWindow *getFrame() const;
