@@ -13,6 +13,8 @@
 #include "yxapp.h"
 #include "prefs.h"
 #include "yprefs.h"
+#include "keysyms.h"
+#include <ctype.h>
 
 struct ZItem {
     int prio;
@@ -344,6 +346,8 @@ public:
                 return i;
         return -1;
     }
+
+    int nextKey(KeySym key) override;
 };
 
 void WindowItemsCtrlr::updateList() {
@@ -437,15 +441,46 @@ void WindowItemsCtrlr::sort() {
         fLastItem.reset();
 }
 
+int WindowItemsCtrlr::nextKey(KeySym keysym) {
+    if ((keysym & 0xff) == keysym && isalpha(keysym & 0xff)) {
+        unsigned char lower = tolower((unsigned char) keysym);
+        for (int i = zTarget + 1; i < zList.getCount(); ++i) {
+            ZItem act = zList[i];
+            ClassHint* hint = act.client->classHint();
+            if (hint) {
+                if (hint->res_class && lower ==
+                    tolower((unsigned char) *hint->res_class)) {
+                    return i;
+                }
+                if (hint->res_name && lower ==
+                    tolower((unsigned char) *hint->res_name)) {
+                    return i;
+                }
+            }
+        }
+    }
+    return -1;
+}
+
+Switcher* Switcher::newSwitchWindow(YWindow* parent, ISwitchItems* items,
+                                    bool verticalStyle) {
+    return new SwitchWindow(parent, items, verticalStyle);
+}
+
 SwitchWindow::SwitchWindow(YWindow *parent, ISwitchItems *items,
                            bool verticalStyle):
-    YPopupWindow(parent),
+    Switcher(parent),
     zItems(items ? items : new WindowItemsCtrlr),
     m_verticalStyle(verticalStyle),
-    m_oldMenuMouseTracking(menuMouseTracking),
     m_hlItemFromMotion(-1),
     m_hintAreaStart(0),
     m_hintAreaStep(1),
+    m_hintAreaFirst(0),
+    m_hintAreaLimit(1),
+    m_hintAreaBanks(1),
+    m_hintFirstBank(0),
+    m_hintAreaOther(0),
+    m_hintOtherEnds(0),
     fWorkspace(WorkspaceInvalid),
     switchFg(&clrQuickSwitchText),
     switchBg(&clrQuickSwitch),
@@ -501,92 +536,121 @@ SwitchWindow::~SwitchWindow() {
 }
 
 void SwitchWindow::resize(int xiscreen, bool reposition) {
-    int dx, dy;
-    unsigned dw, dh;
-
-    desktop->getScreenGeometry(&dx, &dy, &dw, &dh, xiscreen);
+    const int zCount = zItems->getCount();
+    const int active = zItems->getActiveItem();
+    const int vMargins = 2 * max(quickSwitchVMargin, quickSwitchIMargin);
+    const YRect geom(desktop->getScreenGeometry(xiscreen));
+    const int dx = geom.xx, dy = geom.yy, dw = int(geom.ww), dh = int(geom.hh);
+    const int maxWidth = dw * 6/7;
 
     MSG(("got geometry for %d: %d %d %d %d", xiscreen, dx, dy, dw, dh));
 
-    mstring cTitle = zItems->getTitle(zItems->getActiveItem());
-
-    int aWidth =
-        quickSwitchSmallWindow ?
-        (int) dw * 1/3
-        : (m_verticalStyle ? (int) dw * 2/5 : (int) dw * 3/5);
-
-    int tWidth = 0;
-    if (quickSwitchMaxWidth && switchFont) {
+    int assumeWidth = quickSwitchSmallWindow ? dw * 1/3 :
+                             m_verticalStyle ? dw * 2/5 : dw * 3/5;
+    int txtWidth = min(maxWidth, 10 * int(YIcon::largeSize()));
+    if (switchFont) {
+        const int limit = min(zCount, 99);
         const int space = int(switchFont->textWidth(" "));
-        const int zCount = zItems->getCount();
-        for (int i = 0; i < zCount; i++) {
-            mstring title = zItems->getTitle(i);
-            if (title.nonempty()) {
-                int oWidth = (int) switchFont->textWidth(title) + space;
-                if (tWidth < oWidth)
-                    tWidth = oWidth;
+        mstring a = zItems->getTitle(active), b, c;
+        for (int i = 0; i < limit; i++) {
+            if (i != active) {
+                mstring title = zItems->getTitle(i);
+                if (title.length() > a.length()) {
+                    c = b, b = a, a = title;
+                }
+                else if (title.length() > b.length()) {
+                    if (title != a)
+                        c = b, b = title;
+                }
+                else if (title.length() > c.length()) {
+                    if (title != a && title != b)
+                        c = title;
+                }
+            }
+        }
+        for (mstring d : { a, b, c } ) {
+            if (d.nonempty()) {
+                int titleWidth = switchFont->textWidth(d) + 2*space;
+                if (txtWidth < titleWidth)
+                    txtWidth = titleWidth;
             }
         }
     }
-    else if (cTitle.nonempty() && switchFont) {
-        tWidth = switchFont->textWidth(cTitle);
-    }
 
     if (m_verticalStyle || !quickSwitchAllIcons)
-        tWidth += 2 * quickSwitchIMargin + YIcon::largeSize() + 3;
-    if (tWidth > aWidth)
-        aWidth = tWidth;
+        txtWidth += 2 * quickSwitchIMargin + YIcon::largeSize() + 3;
+    if (assumeWidth < txtWidth)
+        assumeWidth = txtWidth;
 
-    int w = aWidth;
+    int w = assumeWidth;
     int h = switchFont ? switchFont->height() : 1;
-    int const mWidth(dw * 6/7);
-    const int vMargins = 2 * max(quickSwitchVMargin, quickSwitchIMargin);
 
     if (m_verticalStyle) {
-        w = aWidth;
-        if (w >= mWidth)
-            w = mWidth;
-        w += quickSwitchSepSize;
-
+        m_hintAreaBanks = 1;
         m_hintAreaStep = YIcon::largeSize() + quickSwitchIMargin;
         m_hintAreaStart = quickSwitchVMargin - quickSwitchIBorder;
-        h = zItems->getCount() * m_hintAreaStep;
-        if (h + vMargins > int(dh))
-            h = (unsigned(dh) - vMargins) / m_hintAreaStep * m_hintAreaStep;
-    } else {
-
-        int iWidth =
-            zItems->getCount() * (YIcon::largeSize() + 2 * quickSwitchIMargin) +
-            (quickSwitchHugeIcon ? YIcon::hugeSize() - YIcon::largeSize() : 0);
-
-        int const iHeight =
-            (quickSwitchHugeIcon ?
-             YIcon::hugeSize() : YIcon::largeSize()) + quickSwitchIMargin * 2;
-
-        if (iWidth > aWidth)
-            aWidth = iWidth;
-
-        if (quickSwitchAllIcons) {
-            if (aWidth > w)
-                w = aWidth;
+        h = zCount * m_hintAreaStep;
+        const int vertical = (dh - (reposition ? 0 : (y() - dy)));
+        if (h > vertical - m_hintAreaStep - vMargins) {
+            h = (vertical - m_hintAreaStep - vMargins)
+                / m_hintAreaStep * m_hintAreaStep;
+            if (w * 3/2 < maxWidth) {
+                int n = h / m_hintAreaStep;
+                m_hintAreaBanks = max(2, min(maxWidth / txtWidth,
+                                      (zCount + n - 1) / n));
+                w = min(maxWidth, m_hintAreaBanks * txtWidth);
+                int k = (zCount + (m_hintAreaBanks - 1))
+                        / m_hintAreaBanks * m_hintAreaStep;
+                if (h > k)
+                    h = k;
+            }
         }
-        if (w >= mWidth)
-            w = mWidth;
+
+        if (w > maxWidth)
+            w = maxWidth;
+        w += quickSwitchSepSize * m_hintAreaBanks;
+    } else {
+        m_hintAreaBanks = 1;
+        int perIcon = YIcon::largeSize() + 2 * quickSwitchIMargin;
+        int perHuge = quickSwitchHugeIcon
+                    ? YIcon::hugeSize() - YIcon::largeSize() : 0;
+        int iconWidth = zCount * perIcon + perHuge;
+        int iconHeight = perIcon + perHuge;
+
+        if (assumeWidth < iconWidth)
+            w = iconWidth;
+        else if (assumeWidth > max(iconWidth, txtWidth))
+            w = max(max(iconWidth, txtWidth), dw * 1/3);
+        else
+            w = assumeWidth;
 
         if (quickSwitchAllIcons)
-            h += quickSwitchSepSize + iHeight;
-        else {
-            if (iHeight > h)
-                h = iHeight;
+            h += quickSwitchSepSize + iconHeight;
+        else if (h < iconHeight)
+            h = iconHeight;
+
+        if (w > maxWidth) {
+            int num = max(1,
+                      int(maxWidth - perHuge - 2*quickSwitchHMargin) / perIcon);
+            int banks = (zCount + (num - 1)) / num;
+            num = (zCount + (banks - 1)) / banks;
+            w = num * perIcon + perHuge + 2*quickSwitchHMargin;
+            int maxBanks = (dh - vMargins - perHuge - perIcon
+                            - quickSwitchSepSize) / perIcon;
+            m_hintAreaBanks = max(1, min(banks, maxBanks));
+        }
+        if (w > maxWidth) {
+            w = maxWidth;
+        }
+        if (m_hintAreaBanks > 1) {
+            h += (m_hintAreaBanks - 1) * iconHeight;
         }
     }
     h += vMargins;
-    w += quickSwitchHMargin * 2;
+    w += 2*quickSwitchHMargin;
 
     if (reposition) {
-        setGeometry(YRect(dx + ((dw - w) >> 1),
-                          dy + ((dh - h) >> 1),
-                          w, h));
+        setGeometry(YRect(dx + (dw - w)/2, dy + (dh - h)/2, w, h));
     } else {
         setSize(w, h);
     }
@@ -629,30 +693,28 @@ void SwitchWindow::paint(Graphics &g, const YRect &/*r*/) {
 }
 
 void SwitchWindow::paintHorizontal(Graphics &g) {
-    if (zItems->getActiveItem() >= 0) {
-        int tOfs(0);
-        int iconSize = quickSwitchHugeIcon ? YIcon::hugeSize() : YIcon::largeSize();
+    const int active = zItems->getActiveItem();
+    if (active >= 0) {
+        const int iconSize = quickSwitchHugeIcon
+                           ? YIcon::hugeSize()
+                           : YIcon::largeSize();
+        int tOfs = 0;
 
-        ref<YIcon> icon;
-        if (!quickSwitchAllIcons &&
-                (icon = zItems->getIcon(zItems->getActiveItem())) != null) {
-            int iconWidth = iconSize, iconHeight = iconSize;
+        ref<YIcon> icon = zItems->getIcon(active);
+        if ( !quickSwitchAllIcons && icon != null) {
+            const int iconWidth = iconSize, iconHeight = iconSize;
+            if (quickSwitchTextFirst) {
+                icon->draw(g,
+                           width() - iconWidth - quickSwitchIMargin,
+                           (height() - iconHeight - quickSwitchIMargin) / 2,
+                           iconSize);
+            } else {
+                icon->draw(g,
+                           quickSwitchIMargin,
+                           (height() - iconHeight - quickSwitchIMargin) / 2,
+                           iconSize);
 
-            if (icon != null) {
-                if (quickSwitchTextFirst) {
-                    icon->draw(g,
-                               width() - iconWidth - quickSwitchIMargin,
-                               (height() - iconHeight - quickSwitchIMargin) / 2,
-                               iconSize);
-                } else {
-                    icon->draw(g,
-                               quickSwitchIMargin,
-                               (height() - iconHeight - quickSwitchIMargin) / 2,
-                               iconSize);
-
-                    tOfs = iconWidth + quickSwitchIMargin
-                        + quickSwitchSepSize;
-                }
+                tOfs = iconWidth + quickSwitchIMargin + quickSwitchSepSize;
             }
 
             if (quickSwitchSepSize) {
@@ -668,27 +730,25 @@ void SwitchWindow::paintHorizontal(Graphics &g) {
         }
 
         g.setColor(switchFg);
-        if (switchFont) {
-            g.setFont(switchFont);
-        }
+        g.setFont(switchFont);
 
-        mstring cTitle = zItems->getTitle(zItems->getActiveItem());
+        mstring cTitle = zItems->getTitle(active);
         if (cTitle != null && switchFont) {
             const int x = max((width() - tOfs -
                                switchFont->textWidth(cTitle)) >> 1, 0U) + tOfs;
             const int y(quickSwitchAllIcons
                         ? quickSwitchTextFirst
-                        ? quickSwitchVMargin + switchFont->ascent()
+                        ? quickSwitchVMargin + switchFont->ascent() + 2
                         : height() - quickSwitchVMargin - switchFont->descent()
                         : ((height() + switchFont->height()) >> 1) -
-                        switchFont->descent());
+                            switchFont->descent());
 
             g.drawChars(cTitle, x, y);
 
             if (quickSwitchAllIcons && quickSwitchSepSize) {
-                int const h(quickSwitchVMargin + iconSize +
-                            quickSwitchIMargin * 2 +
-                            quickSwitchSepSize / 2);
+                int const h(quickSwitchVMargin +
+                            (iconSize + 2*quickSwitchIMargin) * m_hintAreaBanks
+                            + quickSwitchSepSize / 2);
                 int const y(quickSwitchTextFirst ? height() - h : h);
 
                 g.setColor(switchBg->darker());
@@ -703,40 +763,49 @@ void SwitchWindow::paintHorizontal(Graphics &g) {
                          YIcon::largeSize() : 0);
             int const dx(YIcon::largeSize() + 2 * quickSwitchIMargin);
 
-            const int visIcons((width() - 2 * quickSwitchHMargin) / dx);
-            int curIcon(-1);
+            const int bankSize = (width() - 2 * quickSwitchHMargin - ds) / dx;
 
-            int const y(quickSwitchTextFirst
-                        ? height() - quickSwitchVMargin - iconSize - quickSwitchIMargin + ds / 2
-                        : quickSwitchVMargin + ds + quickSwitchIMargin - ds / 2);
+            int y = quickSwitchTextFirst
+                  ? quickSwitchVMargin
+                    + (switchFont ? switchFont->height() : 1)
+                    + quickSwitchSepSize
+                    + quickSwitchIMargin + ds / 2
+                  : quickSwitchVMargin + ds + quickSwitchIMargin - ds / 2;
 
             YColor frameColor = switchHl ? switchHl : switchBg->brighter();
             g.setColor(frameColor);
 
-            const int off(max(1 + curIcon - visIcons, 0));
-            const int end(off + visIcons);
+            const int zCount = zItems->getCount();
 
-            int x((width() - min(visIcons, zItems->getCount()) * dx - ds) /  2 +
-                  quickSwitchIMargin);
+            int x = (width() - min(bankSize, zCount) * dx - ds) / 2 +
+                     quickSwitchIMargin;
 
             m_hintAreaStart = x;
             m_hintAreaStep = dx;
+            m_hintAreaFirst = bankSize * m_hintFirstBank;
+            m_hintAreaLimit = min(zCount, bankSize *
+                                  (m_hintFirstBank + m_hintAreaBanks));
+            m_hintAreaOther = m_hintOtherEnds = y;
 
-            for (int i = 0, zCount = zItems->getCount (); i < zCount; i++) {
-                if (i >= off && i < end) {
+            int xcopy = x;
+            for (int b = 0; b < m_hintAreaBanks; ++b) {
+                int first = (b + m_hintFirstBank) * bankSize;
+                int limit = min((b + 1) * bankSize, zCount);
+                x = xcopy;
+                for (int i = first; i < limit; i++) {
                     ref<YIcon> icon = zItems->getIcon(i);
                     if (icon != null) {
                         if (i == m_hlItemFromMotion &&
-                            i != zItems->getActiveItem())
+                            i != active)
                         {
                             g.setColor(frameColor.darker());
                             g.drawRect(x - quickSwitchIBorder,
                                     y - quickSwitchIBorder - ds / 2,
-                                    iconSize + 2 * quickSwitchIBorder,
+                                    YIcon::largeSize() + 2 * quickSwitchIBorder,
                                     iconSize + 2 * quickSwitchIBorder);
                             g.setColor(frameColor);
                         }
-                        if (i == zItems->getActiveItem()) {
+                        if (i == active) {
                             if (quickSwitchFillSelection)
                                 g.fillRect(x - quickSwitchIBorder,
                                         y - quickSwitchIBorder - ds / 2,
@@ -751,30 +820,69 @@ void SwitchWindow::paintHorizontal(Graphics &g) {
                             if (icon != null) {
                                 icon->draw(g, x, y - ds / 2, iconSize);
                             }
+                            x += ds;
                         } else {
                             icon->draw(g, x, y, YIcon::largeSize());
                         }
-                        x += ds;
+                        x += dx;
+                        if (quickSwitchGroupWorkspaces && i + 1 < limit &&
+                            zItems->getWorkspace(i) != zItems->getWorkspace(i+1))
+                        {
+                            int xoffset = x - quickSwitchIMargin;
+                            int y2 = (i == active || i + 1 == active)
+                                     ? ds / 4 : 0;
+                            g.setColor(switchBg->darker());
+                            g.drawLine(xoffset - 1, y - y2 - quickSwitchIMargin,
+                                       xoffset - 1, y + y2 + dx);
+                            g.setColor(switchBg->brighter());
+                            g.drawLine(xoffset, y - y2 - quickSwitchIMargin,
+                                       xoffset, y + y2 + dx);
+                            g.setColor(frameColor);
+                        }
                     }
-                    x += dx;
                 }
+                y += dx + ds;
             }
+            m_hintOtherEnds = y;
         }
     }
 }
 
 int SwitchWindow::hintedItem(int x, int y)
 {
-    int ends = m_hintAreaStart + m_hintAreaStep * zItems->getCount();
     if (m_verticalStyle) {
+        int count = (int(height()) - m_hintAreaStart - quickSwitchVMargin)
+                  / m_hintAreaStep;
+        int ends = m_hintAreaStart + m_hintAreaStep * count;
         if (x >= 0 && x < int(width()) &&
-            y >= m_hintAreaStart && y < ends)
-            return (y - m_hintAreaStart) / m_hintAreaStep;
+            y >= m_hintAreaStart && y < ends) {
+            int i = (y - m_hintAreaStart) / m_hintAreaStep + m_hintAreaFirst;
+            i += count * (x / (int(width()) / m_hintAreaBanks));
+            if (i >= 0 && i < zItems->getCount())
+                return i;
+        }
     }
-    else if (quickSwitchAllIcons && !quickSwitchHugeIcon) {
-        if (y >= 0 && y < int(height()) &&
-            x >= m_hintAreaStart && x < ends)
-            return (x - m_hintAreaStart) / m_hintAreaStep;
+    else if (quickSwitchAllIcons) {
+        int iconsize = YIcon::largeSize();
+        int active = zItems->getActiveItem();
+        int ds = quickSwitchHugeIcon ? YIcon::hugeSize() - iconsize : 0;
+        int count = (int(width()) - 2*quickSwitchHMargin - ds) / m_hintAreaStep;
+        int ends = m_hintAreaStart + m_hintAreaStep * count;
+        if (y >= m_hintAreaOther - ds/2 &&
+            y < m_hintOtherEnds + iconsize &&
+            x >= m_hintAreaStart && x < ends + ds) {
+            int b = (y - m_hintAreaOther + ds/2) / (m_hintAreaStep + ds);
+            int o = (x - m_hintAreaStart) / m_hintAreaStep;
+            int i = (b + m_hintFirstBank) * count + o;
+            if (i > active && quickSwitchHugeIcon &&
+                active / count == b + m_hintFirstBank) {
+                o = (x - m_hintAreaStart - ds) / m_hintAreaStep;
+                int t = (b + m_hintFirstBank) * count + o;
+                i = (t >= active) ? t : active;
+            }
+            if (i >= 0 && i < zItems->getCount())
+                return i;
+        }
     }
     return -2;
 }
@@ -788,22 +896,33 @@ void SwitchWindow::handleMotion(const XMotionEvent& motion) {
 }
 
 void SwitchWindow::paintVertical(Graphics &g) {
-    // NOTE: quickSwitchHugeIcon not supported in vertical mode. Tried that, looks creepy, not nice (04d53238@code7r)
-    const int iconSize = /* quickSwitchHugeIcon ? YIcon::hugeSize() : */ YIcon::largeSize();
+    const int iconSize = YIcon::largeSize();
+    const int active = zItems->getActiveItem();
+    const int maxWid = width() - 2; // reduce due to 3D edge
+    const int bankWidth = maxWid / m_hintAreaBanks;
+    const int itemWidth = bankWidth - 2*quickSwitchHMargin;
+    const int frameWid = itemWidth + 2*quickSwitchIBorder;
+    const int frameHght = iconSize + 2*quickSwitchIBorder;
+    const int vMargins = 2 * max(quickSwitchVMargin, quickSwitchIMargin);
+    const int bankCount = (height() - vMargins) / m_hintAreaStep;
 
-    if (zItems->getActiveItem() >= 0) {
-        const int maxWid = width() - 2; // reduce due to 3D edge
-        const int contentX = quickSwitchHMargin;
-        const int titleX = quickSwitchTextFirst ? contentX
-                : contentX + iconSize + quickSwitchSepSize;
-        const int itemWidth = maxWid - quickSwitchHMargin*2;
+    for (int bank = 0; bank < m_hintAreaBanks && 0 <= active; ++bank) {
+        const int bankStart = bank * bankWidth;
+        const int contentX = bankStart + quickSwitchHMargin;
+        const int titleX = quickSwitchTextFirst ? contentX :
+                           contentX + iconSize + quickSwitchSepSize;
         const int frameX = contentX - quickSwitchIBorder;
-        const int frameWid = itemWidth + 2 * quickSwitchIBorder;
-        const int frameHght = iconSize + 2*quickSwitchIBorder;
-        const int strWid = itemWidth - iconSize - quickSwitchSepSize - 2*quickSwitchHMargin;
+        const int strWid = itemWidth - iconSize - quickSwitchSepSize
+                           - 2*quickSwitchHMargin;
         const int sepX = quickSwitchTextFirst
-                ? maxWid - iconSize - quickSwitchSepSize/2 - 1
-                        :  contentX + iconSize + quickSwitchSepSize/2 - 1;
+                         ? bankStart + bankWidth
+                                    - iconSize - quickSwitchSepSize/2 - 1
+                         : contentX + iconSize + quickSwitchSepSize/2 - 1;
+        const int first = (bank + m_hintFirstBank) * bankCount;
+        const int zCount = min(first + bankCount, zItems->getCount()) - first;
+        m_hintAreaFirst = m_hintFirstBank * bankCount;
+        m_hintAreaLimit = first + zCount;
+        m_hintAreaOther = m_hintOtherEnds = 0;
 
         int contentY = quickSwitchVMargin + quickSwitchIBorder;
 
@@ -811,16 +930,18 @@ void SwitchWindow::paintVertical(Graphics &g) {
             g.setFont(switchFont);
         }
         g.setColor(switchFg);
-        for (int i = 0, zCount = zItems->getCount(); i < zCount; i++) {
+        for (int i = first; i - first < zCount; i++) {
             if (contentY + frameHght > int(quickSwitchIBorder + height()))
                 break;
             if (i > 0 && zItems->getWorkspace(i) != zItems->getWorkspace(i-1)) {
+                int x0 = 1 + bank * bankWidth;
+                int x1 = x0 + bankWidth - 2;
                 g.setColor(switchBg->darker());
-                g.drawLine(1, contentY - 4, width() - 2, contentY - 4);
+                g.drawLine(x0, contentY - 4, x1, contentY - 4);
                 g.setColor(switchBg->brighter());
-                g.drawLine(1, contentY - 3, width() - 2, contentY - 3);
+                g.drawLine(x0, contentY - 3, x1, contentY - 3);
             }
-            if (i == zItems->getActiveItem()) {
+            if (i == active) {
                 g.setColor(switchMbg);
                 g.fillRect(frameX, contentY-quickSwitchIBorder, frameWid, frameHght);
                 g.setColor(switchMfg);
@@ -837,12 +958,12 @@ void SwitchWindow::paintVertical(Graphics &g) {
             ref<YIcon> icon = zItems->getIcon(i);
             if (icon != null) {
                 int iconX = quickSwitchTextFirst
-                        ? width() - quickSwitchHMargin - iconSize
+                        ? bankStart + bankWidth - quickSwitchHMargin - iconSize
                         : contentX;
                 icon->draw(g, iconX, contentY, iconSize);
             }
 
-            if (i == m_hlItemFromMotion && i != zItems->getActiveItem())
+            if (i == m_hlItemFromMotion && i != active)
             {
                 g.setColor(switchMbg);
                 g.drawRect(frameX, contentY-quickSwitchIBorder, frameWid, frameHght);
@@ -850,10 +971,18 @@ void SwitchWindow::paintVertical(Graphics &g) {
             contentY += m_hintAreaStep;
         }
         if (quickSwitchSepSize) {
+            const int margin = quickSwitchVMargin;
+            const int bottom = height() - margin - 2;
             g.setColor(switchBg->darker());
-            g.drawLine(sepX + 0, 1, sepX + 0, height() - 2);
+            g.drawLine(sepX + 0, margin, sepX + 0, bottom);
             g.setColor(switchBg->brighter());
-            g.drawLine(sepX + 1, 1, sepX + 1, height() - 2);
+            g.drawLine(sepX + 1, margin, sepX + 1, bottom);
+        }
+        if (bank) {
+            g.setColor(switchBg->darker());
+            g.drawLine(bankStart - 1, 1, bankStart - 1, height() - 3);
+            g.setColor(switchBg->brighter());
+            g.drawLine(bankStart, 1, bankStart, height() - 3);
         }
     }
 }
@@ -861,8 +990,7 @@ void SwitchWindow::paintVertical(Graphics &g) {
 void SwitchWindow::begin(bool zdown, unsigned mods, char* wmclass) {
     bool change = zItems->setWMClass(wmclass);
     modsDown = KEY_MODMASK(mods);
-    m_oldMenuMouseTracking = menuMouseTracking;
-    menuMouseTracking = true;
+    setPointerMotion(true);
 
     if (zItems->isEmpty() || change || !quickSwitchPersistence ||
         (fWorkspace != manager->activeWorkspace() &&
@@ -891,7 +1019,6 @@ void SwitchWindow::activatePopup(int flags) {
 }
 
 void SwitchWindow::deactivatePopup() {
-    menuMouseTracking = m_oldMenuMouseTracking;
     m_hlItemFromMotion = -1;
 }
 
@@ -903,6 +1030,7 @@ void SwitchWindow::displayFocus() {
 void SwitchWindow::destroyedFrame(YFrameWindow *frame) {
     int active = zItems->getActiveItem();
     if (zItems->destroyedItem(frame, nullptr)) {
+        m_hlItemFromMotion = -1;
         if (zItems->isEmpty()) {
             cancel();
         }
@@ -916,6 +1044,7 @@ void SwitchWindow::destroyedFrame(YFrameWindow *frame) {
 void SwitchWindow::destroyedClient(YFrameClient* client) {
     int active = zItems->getActiveItem();
     if (zItems->destroyedItem(nullptr, client)) {
+        m_hlItemFromMotion = -1;
         if (zItems->isEmpty()) {
             cancel();
         }
@@ -945,12 +1074,43 @@ YFrameWindow* SwitchWindow::current() {
     return zItems->current();
 }
 
+
+int add_modulo(int active, int delta, int count) {
+    return ((active + delta) % count + count) % count;
+}
+
 void SwitchWindow::target(int delta) {
     if (delta) {
         m_hlItemFromMotion = -1;
         int active = zItems->getActiveItem();
         int count = zItems->getCount();
-        int index = (1 < count) ? (active + delta + count) % count : 0;
+        int index = (1 < count) ? add_modulo(active, delta, count) : 0;
+        int banksize = 0;
+
+        if (m_verticalStyle) {
+            banksize = (int(height()) - m_hintAreaStart) / m_hintAreaStep;
+        }
+        else if (quickSwitchAllIcons) {
+            int dx = YIcon::largeSize() + 2*quickSwitchIMargin;
+            int ds = quickSwitchHugeIcon ? YIcon::hugeSize() -
+                      YIcon::largeSize() : 0;
+            banksize = (width() - 2*quickSwitchHMargin - ds) / dx;
+        }
+
+        if (banksize) {
+            if (index < active) {
+                while (index < m_hintFirstBank * banksize) {
+                    m_hintFirstBank--;
+                }
+            }
+            else {
+                while (index >=
+                       (m_hintFirstBank + m_hintAreaBanks) * banksize) {
+                    m_hintFirstBank++;
+                }
+            }
+        }
+
         zItems->setTarget(index);
         displayFocus();
     }
@@ -958,6 +1118,7 @@ void SwitchWindow::target(int delta) {
 
 bool SwitchWindow::handleKey(const XKeyEvent &key) {
     KeySym k = keyCodeToKeySym(key.keycode);
+    k = mapKeypad(k);
     if (key.type == KeyPress) {
         keyPressed = k;
         if (isKey(key)) {
@@ -988,17 +1149,55 @@ bool SwitchWindow::handleKey(const XKeyEvent &key) {
                 cancel();
             }
         }
-        else if (k == XK_Down && m_verticalStyle) {
-            target(+1);
+        else if (k == XK_Down) {
+            if (m_verticalStyle) {
+                target(+1);
+            } else {
+                int dx = YIcon::largeSize() + 2*quickSwitchIMargin;
+                int ds = quickSwitchHugeIcon ? YIcon::hugeSize() -
+                          YIcon::largeSize() : 0;
+                int count = (width() - 2*quickSwitchHMargin - ds) / dx;
+                int activ = zItems->getActiveItem();
+                if (activ + count < zItems->getCount()) {
+                    target(+count);
+                }
+            }
         }
-        else if (k == XK_Up && m_verticalStyle) {
-            target(-1);
+        else if (k == XK_Up) {
+            if (m_verticalStyle) {
+                target(-1);
+            } else {
+                int dx = YIcon::largeSize() + 2*quickSwitchIMargin;
+                int ds = quickSwitchHugeIcon ? YIcon::hugeSize() -
+                          YIcon::largeSize() : 0;
+                int count = (width() - 2*quickSwitchHMargin - ds) / dx;
+                int activ = zItems->getActiveItem();
+                if (activ >= count) {
+                    target(-count);
+                }
+            }
         }
-        else if (k == XK_Right && !m_verticalStyle) {
-            target(+1);
+        else if (k == XK_Right) {
+            if ( !m_verticalStyle) {
+                target(+1);
+            } else {
+                int count = (int(height()) - m_hintAreaStart) / m_hintAreaStep;
+                int activ = zItems->getActiveItem();
+                if (activ + count < zItems->getCount()) {
+                    target(+count);
+                }
+            }
         }
-        else if (k == XK_Left && !m_verticalStyle) {
-            target(-1);
+        else if (k == XK_Left) {
+            if ( !m_verticalStyle) {
+                target(-1);
+            } else {
+                int count = (int(height()) - m_hintAreaStart) / m_hintAreaStep;
+                int activ = zItems->getActiveItem();
+                if (activ >= count) {
+                    target(-count);
+                }
+            }
         }
         else if (k == XK_End) {
             int num = zItems->getCount();
@@ -1006,19 +1205,37 @@ bool SwitchWindow::handleKey(const XKeyEvent &key) {
             target(num - 1 - act);
         }
         else if (k == XK_Home) {
+            m_hintFirstBank = 0;
             target(-zItems->getActiveItem());
         }
+        else if (k == XK_Next) {
+            int num = zItems->getCount();
+            int act = zItems->getActiveItem();
+            target(min(act + 10, num - 1) - act);
+        }
+        else if (k == XK_Prior) {
+            int act = zItems->getActiveItem();
+            target(max(act - 10, 0) - act);
+        }
         else if (k == XK_Delete) {
             zItems->destroyTarget();
         }
         else if (k >= '1' && k <= '9') {
-            int index = int(k - '1');
-            if (index < zItems->getCount())
-                target(index - zItems->getActiveItem());
+            if (0 <= m_hintAreaFirst && m_hintAreaLimit <= zItems->getCount()) {
+                int index = int(k - '1') + m_hintAreaFirst;
+                if (index < zItems->getCount())
+                    target(index - zItems->getActiveItem());
+            }
         }
         else if (zItems->isKey(key) && !modDown(key.state)) {
             accept();
         }
+        else {
+            int i = zItems->nextKey(k);
+            if (i >= 0) {
+                target(i - zItems->getActiveItem());
+            }
+        }
     }
     else if (key.type == KeyRelease) {
         if ((isKey(key) && !modDown(key.state)) || isModKey(key.keycode)) {
@@ -1103,6 +1320,18 @@ void SwitchWindow::handleButton(const XButtonEvent &button) {
             zItems->destroyTarget();
         }
         break;
+    case Button3:
+        m_hlItemFromMotion = -1;
+        if ((hint = hintedItem(button.x, button.y)) >= 0) {
+            target(hint - zItems->getActiveItem());
+            YFrameWindow* frame = zItems->current();
+            if (frame) {
+                frame->popupSystemMenu(this, button.x_root, button.y_root,
+                                       YPopupWindow::pfCanFlipVertical |
+                                       YPopupWindow::pfCanFlipHorizontal);
+            }
+        }
+        break;
     case Button4:
         target(-1);
         break;
